//----------------------------------------------------------------------------
// SVG Viewer - Version 1.0
// A library of SVG renderer
// Copyright (C) 2008 Witold Jaworski
// Contact:	witold.jaworski@airplanes3d.net
//			http://airplanes3d.net
// 
// Contains elements from:
//			AGG - a graphic library (Copyright Maxim Shemanarev, mcseem@antigrain.com)
//          http://antigrain.com
// Uses functions from:
//          Expat - a XML parser library (Copyright James Clark)
// 
// SVG Viewer is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// SVG Viewer is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with SVG Viewer; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
// MA 02110-1301, USA.
//----------------------------------------------------------------------------
#include "svg_exception.h"
#include "svg_basics.h"

namespace svg
{
//////////////////////////////////////////////////////////////////////
// Some other useful stuff:
//////////////////////////////////////////////////////////////////////
//Helper function - just to compare two rectangles
 bool is_equal(const rectangle &r1, const rectangle &r2)
{
	 return agg::is_equal_eps(r1.x1, r2.x1, SVG_EPSILON) 
		   && 
		   agg::is_equal_eps(r1.y1, r2.y1, SVG_EPSILON) 
		   &&
		   agg::is_equal_eps(r1.x2, r2.x2, SVG_EPSILON) 
		   && 
		   agg::is_equal_eps(r1.y2, r2.y2, SVG_EPSILON); 
}

//Helper function - checks, if the rectangle has zero width or height 
bool has_no_area(const rectangle &rec)
{
	return agg::is_equal_eps(rec.x1, rec.x2, SVG_EPSILON) 
		   ||
		   agg::is_equal_eps(rec.y1, rec.y2, SVG_EPSILON); 
}

//Helper function - checks if the transformation is just a 
//plain translation, without scaling, shear or rotation
//Arguments
//	trans:	transformation to check
bool is_translation_only(transform trans)
{
	transform identity = transform();
	trans.tx = trans.ty = 0.0; //clear the translation
	return (identity == trans); //without translation trans
								//should be equalt to identity
}

//Helper function - checks if the transformation is empty
//Arguments
//	trans:	transformation to check
bool is_empty(transform trans)
{
	transform empty = transform(0,0,0,0,0,0);
	return (empty == trans); 
}

//Returns true, when the inner rectangle is inside outer
//Arguments:
//	inner:	the presumably smaller rectangle
//	outer:	the presumably larger rectangle
bool is_inside(const rectangle &inner, const rectangle &outer)
{ 
	return is_equal(agg::unite_rectangles(outer,inner),outer);
}

//Returns true, when the both colors are the same
//Arguments:
//	c1,c2:	colors to compare
bool is_equal(rgba8 &c1, rgba8 &c2)
{
	return (c1.r == c2.r && c1.g == c2.g && c1.b == c2.b && c1.a == c2.a); 
}

//----------------------------------------------------------------------------
} // namespace svg