//----------------------------------------------------------------------------
// SVG Viewer - Version 1.0
// A library of SVG renderer
// Copyright (C) 2008 Witold Jaworski
// Contact:	witold.jaworski@airplanes3d.net
//			http://airplanes3d.net
// 
// Contains elements from:
//			AGG - a graphic library (Copyright Maxim Shemanarev, mcseem@antigrain.com)
//          http://antigrain.com
// Uses functions from:
//          Expat - a XML parser library (Copyright James Clark)
// 
// SVG Viewer is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// SVG Viewer is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with SVG Viewer; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
// MA 02110-1301, USA.
//----------------------------------------------------------------------------

#ifndef SVG_DASH_INCLUDED
#include "svg_element.h"
#include "svg_req_render.h"
#define SVG_DASH_INCLUDED

namespace svg
{
//////////////////////////////////////////////////////////////////////
// dash: a helper element (non-existent in the source file) to store
//		 the array of a dash pattern definition spans (and gaps) 
//////////////////////////////////////////////////////////////////////
class dash : public element 
{
	public:
		//////////////////////////////////////////////////////////////////////
		// constructor and overriden virtual methods
		//////////////////////////////////////////////////////////////////////

		dash(element *parent, unsigned level): element(parent, level) {}

		//Static constructor - used by the parser to create a new dash pattern
		//Arguments:
		//	prs:	parser;
		//	parent:	the element, that will be the parent of this element
		//	level:	the level of nesting in the source XML tree (just parent->level()+1 here)
		//	spec:	the specification of this element - a string, which plays both roles:
		//			the complete specification of the object content, and its name.
		static element *create(parser &prs, element *parent, unsigned level, const char *spec)
		{
			dash *elm = new dash(parent, level);
			char span[SVG_MAX_NAME]; //a buffer for the single span length...
			char *p = (char *) spec;
			while (*p)
			{
				p = prs.get_first_token(p,',',span,sizeof(span));
				elm->on_attribute(prs,"span",span);
			}
			//copy the specification - it will be the name...
			strncpy(elm->m_spec,spec,sizeof(elm->m_spec));
			elm->m_name = elm->m_spec;

			return elm;
		}

		//prefix "ext:" is for elements that have no corresponding element in 
		//the XML SVG document structure
		virtual const char * tag() {return "ext:dash";}; 
														 

		virtual void on_attribute(parser &prs, const char * name, const char *value)
		{
			if (strcmp(name, "span") == 0) //there is no such attribute in the XML 
			{							   //see parser::parse_style() function	
				 prs.store(*this,attribute(attribute::dash_span,prs.to_length(value)));
			}
			else 
			{
				//nothing - this element has not to have any other attributes
			}
		}

		virtual void commit(parser &prs) 
		{	//It MUST have even number of spans (attributes) 
			unsigned spans = m_attribute_last - m_attribute_first + 1; //number of the spans
			if ((spans % 2) != 0)	//ODD number of the spans - according the SVG
			{						//spec. we have to duplicate them:
				handle id;
				for (id = 0; id < spans; id++)
				{	
					attribute *span = prs.get_attribute(id);
					if (span) prs.store(*this,*span);
				}
			}
			element::commit(prs);
		}
		
	public:
		//////////////////////////////////////////////////////////////////////
		// Other methods and fields
		//////////////////////////////////////////////////////////////////////
		//Applies dash attributes to a vertex source 
		//Arguments:
		//	req: the request object
		//	dc:	 the vertex converter (one of conv_dash<>) to be modified
		void apply(req_render &req, req_render::dashed &dc)
		{
			handle id;
			dc.remove_all_dashes();
			for(id = m_attribute_first; id < m_attribute_last; id+= 2)
			{
				attribute *span = req.get_attribute(id);
				attribute *gap	= req.get_attribute(id+1);
				if (span && gap) dc.add_dash(span->val.real, gap->val.real);
			}
		}
		
	protected:
		char m_spec[SVG_MAX_ATTRIB_VALUE];

}; //dash

}; //namespace svg
#endif SVG_DASH_INCLUDED
