//----------------------------------------------------------------------------
// SVG Viewer - Version 1.0
// A library of SVG renderer
// Copyright (C) 2008 Witold Jaworski
// Contact:	witold.jaworski@airplanes3d.net
//			http://airplanes3d.net
// 
// Contains elements from:
//			AGG - a graphic library (Copyright Maxim Shemanarev, mcseem@antigrain.com)
//          http://antigrain.com
// Uses functions from:
//          Expat - a XML parser library (Copyright James Clark)
// 
// SVG Viewer is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// SVG Viewer is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with SVG Viewer; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
// MA 02110-1301, USA.
//----------------------------------------------------------------------------

#ifndef SVG_GAUSSIAN_BLUR_INCLUDED
#include "agg_blur.h"

#include "svg_filter_primitive.h"
#define SVG_GAUSSIAN_BLUR_INCLUDED

namespace svg
{
//////////////////////////////////////////////////////////////////////
// fe_gaussian_blur: an example of real filter primitive.
// It implements the SVG feGaussianBlur primitive.
//////////////////////////////////////////////////////////////////////
class fe_gaussian_blur : public filter_primitive
{
	public:
		//////////////////////////////////////////////////////////////////////
		// constructor and overriden virtual methods
		//////////////////////////////////////////////////////////////////////

		fe_gaussian_blur(element *parent, unsigned level): filter_primitive(parent, level),
		m_rx(-1),
		m_ry(-1) //negative values are treated as "not initialized"
		{}

		virtual const char * tag() {return "feGaussianBlur";}; 
														 
		virtual void on_attribute(parser &prs, const char * name, const char *value)
		{
			if (strcmp(name, "stdDeviation") == 0) 
			{	
				//value can be one or TWO numbers, separated by comma or space
				//("xxxx", or "xxx xxx", or "xxx,xxx"):
				char nmbr[SVG_MAX_NAME]; //a buffer for the single number...
				char separator = strchr(value,',')?',':' '; //check: comma or space?
				char *p = (char *) value;
				while (*p)
				{
					bool relative = false;
					double *target = NULL;
					p = prs.get_first_token(p,separator,nmbr,sizeof(nmbr));

					if(m_rx < 0)			//select, which radius to update
						target = &m_rx;
					else
						target = &m_ry;

					*target = prs.to_length(nmbr, &relative) * 2; //radius of the blur is 
																  //is equal 2*deviation			 
				}
				
				if (m_rx > 0 && m_ry < 0) m_ry = m_rx;
			}
			else 
			{
				filter_primitive::on_attribute(prs,name,value);
			}
		}

	public:

		//override of the base class method:
		virtual handle perform(req_filter &req)
		{
			filter_primitive::perform(req); //always call the base, first!
			
			//do, what you have to do:
			if (m_rx < 0 || m_ry < 0)
			{
				return skip(req); //nothing to do - radius not initialized!
			}
			
			unsigned rx, ry;

			//translate radius into pixel coordinates
			rx = req.prm_to_length(m_rx, 1.0);
			ry = req.prm_to_length(m_ry, 0.0);

			if (rx < 254 && ry < 254) //quick, but not for large radii:
			{
				agg::stack_blur_rgba32(req.get(""), rx, ry);
			}
			else //for any radius, but slower:
			{
				pixfmt_pre &img = req.get(""); 
				m_recursive_blur.blur_x(img,rx);
				m_recursive_blur.blur_y(img,ry);
			}

			//at the end - normal request return:
			return skip(req);
		}

	protected:
		double m_rx, m_ry; //radius of the blur in the x and y direction, as specified 
						   //in the XML source
		
		agg::recursive_blur<agg::rgba8, agg::recursive_blur_calc_rgba<> > m_recursive_blur;

}; //fe_gaussian_blur

}; //namespace svg
#endif SVG_GAUSSIAN_BLUR_INCLUDED
