//----------------------------------------------------------------------------
// SVG Viewer - Version 1.0
// A library of SVG renderer
// Copyright (C) 2008 Witold Jaworski
// Contact:	witold.jaworski@airplanes3d.net
//			http://airplanes3d.net
// 
// Contains elements from:
//			AGG - a graphic library (Copyright Maxim Shemanarev, mcseem@antigrain.com)
//          http://antigrain.com
// Uses functions from:
//          Expat - a XML parser library (Copyright James Clark)
// 
// SVG Viewer is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// SVG Viewer is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with SVG Viewer; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
// MA 02110-1301, USA.
//----------------------------------------------------------------------------

#ifndef SVG_FILTER_PRIMITIVE_INCLUDED
#include "svg_element.h"
#include "svg_req_filter.h"
#define SVG_FILTER_PRIMITIVE_INCLUDED

namespace svg
{
//////////////////////////////////////////////////////////////////////
// filter_primitive: base class for the filter primitives.
// Each	of them has a input and result buffer, and transfers the
// pixels between them
//////////////////////////////////////////////////////////////////////
class filter_primitive : public element 
{
	public:
		//////////////////////////////////////////////////////////////////////
		// constructor and overriden virtual methods
		//////////////////////////////////////////////////////////////////////

		filter_primitive(element *parent, unsigned level): element(parent, level)
		{
			//default buffer names: ""
			m_in[0] = m_in[sizeof(m_in)-1] = 0;
			m_result[0] = m_result[sizeof(m_result)-1] = 0;
		}

		virtual const char * tag() {return "filter_primitive";}; 
														 
		virtual void on_attribute(parser &prs, const char * name, const char *value)
		{
			if (strcmp(name, "in") == 0) 
			{							   
				strncpy(m_in,value,sizeof(m_in)-1);				 
			}
			else 
			if (strcmp(name, "result") == 0) 
			{							   
				strncpy(m_result,value,sizeof(m_result)-1);				 
			}
			else 
			{
				element::on_attribute(prs,name,value);
			}
		}

		//request handler:
		virtual handle on_request(request &req)
		{
			switch(req.type())
			{
			case request::filter:
				{
					return perform((req_filter &) req);
				}

			default: 
				return element::on_request(req);
			}//switch
		}

	public:

		//Function that should be extended later on in the child
		//classes.
		//Arguments:
		//	req:	request with information required by the 
		//			filter primitives to perform their effect
		//Make your own version, but always call this one at
		//the begining:
		virtual handle perform(req_filter &req)
		{
			pixfmt_pre &src = req.get(m_in);
			pixfmt_pre &dst = req.get(m_result);
			//if the source is not the same as destination:
			if (&src != &dst) filter::copy(dst,src,0,0);
			return skip(req);
		}

	protected:
		char	m_in[SVG_MAX_NAME];		//name of the input (may be = "")
		char	m_result[SVG_MAX_NAME]; //name of the output (may be "")

}; //filter_primitive

}; //namespace svg
#endif SVG_FILTER_PRIMITIVE_INCLUDED
