//----------------------------------------------------------------------------
// SVG Viewer - Version 1.0
// A library of SVG renderer
// Copyright (C) 2008 Witold Jaworski
// Contact:	witold.jaworski@airplanes3d.net
//			http://airplanes3d.net
// 
// Contains elements from:
//			AGG - a graphic library (Copyright Maxim Shemanarev, mcseem@antigrain.com)
//          http://antigrain.com
// Uses functions from:
//          Expat - a XML parser library (Copyright James Clark)
// 
// SVG Viewer is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// SVG Viewer is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with SVG Viewer; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
// MA 02110-1301, USA.
//----------------------------------------------------------------------------
#include <stdlib.h>
#include <windows.h>
#include "svg_exception.h"
#include "svg_font_win32.h"

#ifdef SVG_FONT_WIN32_INCLUDED //not defined, when in the compiler defines there is
							   //no "WIN32" def! - see svg_font_win32.h	
namespace svg
{
	//////////////////////////////////////////////////////////////////////
	// constructor and destructor 
	//////////////////////////////////////////////////////////////////////
	//after creation it still requires initialization (call the initialize())
	font_win32::font_win32(element *parent, unsigned level): font(parent, level),
		m_dc(::GetDC(0)), //HWND = 0: desktop handle
		m_feng(m_dc), 
		m_fman(m_feng),
		m_vs(m_fman.path_adaptor()),
		m_bounds(0,0,0,0),
		m_glyph(NULL)
	{
		m_kerning_x = m_kerning_y = 0;
	}
	//--------------------------------------------------------------------
	font_win32::~font_win32()
	{
		::ReleaseDC(0,m_dc); //HWND = 0: desktop handle
	}

	//--------------------------------------------------------------------
	//Called internally by the initialize(spec) function.
	// Arguments:
	//	family:	font family name, like "Arial" or "Times New Roman"
	//	italic: true, when the font has to in the italic version
	//	weight: weight of the font
	void font_win32::initialize(const char *family, bool italic, font::weight_value weight)
	{
		//default values:
		m_feng.height(72.0); //Size is not so important - text is always scaled
							 //just let it to be large enough to be exact
		m_feng.hinting(false); //we are using vector outlines - no hinting needed

		m_feng.flip_y(true);

		//received values:
		m_feng.italic(italic);
		m_feng.weight(weight);

		//create!
		m_feng.create_font(family, agg::glyph_ren_outline);
		m_fman.precache(' ',127);

		//format the internal spec and make it the font's name:
		m_name = specification(m_spec, sizeof(m_spec), family,italic,weight);
	}
	
	//--------------------------------------------------------------------
	//Selects an actual glyph, which corresponds to the code
	//Arguments:
	//	code:			code of the glyph, that has to be selected
	//	apply_kerning:	true, if the kerning has to be used
	void font_win32::select(unicode code, bool apply_kerning)
	{
		//clear the previous data:
		m_code = 0; //nothing!
		m_kerning_x = m_kerning_y = 0;
		m_bounds.x1 = m_bounds.y1 = m_bounds.x2 = m_bounds.y2 = 0;
		m_glyph = m_fman.glyph(code);
		if (m_glyph)
		{
			m_code = code;

			if (apply_kerning) m_fman.add_kerning(&m_kerning_x, &m_kerning_y);
			m_fman.init_embedded_adaptors(m_glyph, m_kerning_x, m_kerning_y);

			agg::rect_i bnd = m_glyph->bounds;

			m_bounds.x1 = bnd.x1;
			m_bounds.y1 = bnd.y1;
			m_bounds.x2 = bnd.x2;
			m_bounds.y2 = bnd.y2;
		}
	}
} // namespace svg
#endif SVG_FONT_WIN32_INCLUDED