//----------------------------------------------------------------------------
// SVG Viewer - Version 1.0
// A library of SVG renderer
// Copyright (C) 2008 Witold Jaworski
// Contact:	witold.jaworski@airplanes3d.net
//			http://airplanes3d.net
// 
// Contains elements from:
//			AGG - a graphic library (Copyright Maxim Shemanarev, mcseem@antigrain.com)
//          http://antigrain.com
// Uses functions from:
//          Expat - a XML parser library (Copyright James Clark)
// 
// SVG Viewer is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// SVG Viewer is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with SVG Viewer; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
// MA 02110-1301, USA.
//----------------------------------------------------------------------------
#ifdef WIN32
#ifndef SVG_FONT_WIN32_INCLUDED
#include "svg_font.h"
#include "agg_font_cache_manager.h"
#include "../font_win32_tt/agg_font_win32_tt.h"
#define SVG_FONT_WIN32_INCLUDED

namespace svg
{
//////////////////////////////////////////////////////////////////////
// font_win32: implementation of a font element for M$ Windows
//////////////////////////////////////////////////////////////////////
class font_win32 : public font 
{
	public:
		//////////////////////////////////////////////////////////////////////
		// constructor and destructor 
		//////////////////////////////////////////////////////////////////////
		//after creation it still requires initialization (call the initialize())
		font_win32(element *parent, unsigned level); 

		virtual ~font_win32();

		//Static constructor - used by the parser to create a new font
		//Arguments:
		//	prs:	parser;
		//	parent:	the element, that will be the parent of this element
		//	level:	the level of nesting in the source XML tree (just parent->level()+1 here)
		//	spec:	the specification of this font ("[style ][variant ][weight ]<size>[ family]")
		static element *create(parser &prs, element *parent, unsigned level, const char *spec)
		{
			font_win32 *elm = new font_win32(parent, level);
			if (elm)
			{
				elm->font::initialize(spec);
			}
			return elm;
		}

	public:
		//////////////////////////////////////////////////////////////////////
		// General information - to be implemented in the inherited class
		//////////////////////////////////////////////////////////////////////
		//Returns true, when the font is oblique
		virtual bool italic() {return m_feng.italic();}; 

		//Returns the weight of the font (from 100 to 900)
		virtual weight_value weight() {return (weight_value) m_feng.weight();};

		//Returns the size (height of a capital letters) of the font
		virtual double size() {return m_feng.height();}; 

		//Returns the name of the font family
		virtual const char *family() {return m_feng.typeface();};
		
		//////////////////////////////////////////////////////////////////////
		// Glyph: selection and geometry
		//////////////////////////////////////////////////////////////////////
		//Selects an actual glyph, which corresponds to the code
		//Arguments:
		//	code:			code of the glyph, that has to be selected
		//	apply_kerning:	true, if the kerning has to be used
		virtual void select(unicode code, bool apply_kerning = false);

		//Returns the vertex source, that contains selected's glyph outline:
		virtual vsource *vertices() {return &m_vs;}

		//Returns the advance of the font direction
		virtual double advance_x() 
			{
				return /* m_feng.advance_x() */(m_glyph)? m_glyph->advance_x : 0.0;
			};
		virtual double advance_y() 
			{
				return /* m_feng.advance_y() */(m_glyph)? m_glyph->advance_y : 0.0;
			};
		//Returns a kernng values for current glyph, and the previous
		virtual double kerning_x() {return -m_kerning_x;};
		virtual double kerning_y() {return -m_kerning_y;};

		//Bounds of the glyph
		virtual const rectangle &bounds() {return m_bounds;};

		//Actual glyph data 
		//virtual const agg::glyph_cache *current() {return m_glyph;};

	protected:
		//////////////////////////////////////////////////////////////////////
		// Other methods and fields
		//////////////////////////////////////////////////////////////////////
		//Called internally by the initialize(spec) function.
		// Arguments:
		//	family:	font family name, like "Arial" or "Times New Roman"
		//	italic: true, when the font has to in the italic version
		//	weight: weight of the font
		//
		//To be implemented in the inheritors classes.
		virtual void initialize(const char *family, bool italic, weight_value weight);

	protected:
		typedef agg::font_engine_win32_tt_int32 font_engine;
		typedef agg::font_cache_manager<font_engine> font_manager;
		
		HDC						m_dc;		//Windows device context
		font_engine				m_feng;
		font_manager			m_fman;
		const agg::glyph_cache	*m_glyph;	//selected glyph
		rectangle				m_bounds;	//bounds of actual glyph 
		double					m_kerning_x, m_kerning_y; //kerning of the actual glyph

		//////////////////////////////////////////////////////////////////////
		// vs_glyph: a wrapper around the path_adaptor, that implements
		//			the vertex_source interface 
		//////////////////////////////////////////////////////////////////////
		typedef font_manager::path_adaptor_type glyph_adaptor;

		class glyph: public vsource
		{
			protected:
				font_manager::path_adaptor_type *m_source;
			public:
				glyph(font_manager::path_adaptor_type &p): m_source(&p) {}
				virtual void rewind(unsigned code) {m_source->rewind(code);}
				virtual unsigned vertex(double* x, double* y) {return m_source->vertex(x, y);}
		} m_vs; 

}; //font_win32

}; //namespace svg
#endif SVG_FONT_WIN32_INCLUDED
#endif WIN32