//----------------------------------------------------------------------------
// SVG Viewer - Version 1.0
// A library of SVG renderer
// Copyright (C) 2008 Witold Jaworski
// Contact:	witold.jaworski@airplanes3d.net
//			http://airplanes3d.net
// 
// Contains elements from:
//			AGG - a graphic library (Copyright Maxim Shemanarev, mcseem@antigrain.com)
//          http://antigrain.com
// Uses functions from:
//          Expat - a XML parser library (Copyright James Clark)
// 
// SVG Viewer is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// SVG Viewer is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with SVG Viewer; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
// MA 02110-1301, USA.
//----------------------------------------------------------------------------

#ifndef SVG_GROUP_INCLUDED
#include "svg_shape.h"
#include "svg_req_render.h"
#define SVG_GROUP_INCLUDED

namespace svg
{
//////////////////////////////////////////////////////////////////////
// group: a group of the shapes 
//	
//////////////////////////////////////////////////////////////////////
class group : public shape 
{
	public:
		//////////////////////////////////////////////////////////////////////
		// constructor and overriden virtual methods
		//////////////////////////////////////////////////////////////////////

		group(element *parent, unsigned level): shape(parent, level) {}

		virtual const char * tag() {return "g";};

		//////////////////////////////////////////////////////////////////////
		// Other methods and fields
		//////////////////////////////////////////////////////////////////////

		virtual bool is_of(const char *cls) 
		{
			if(strcmp("group",cls) == 0)
			{
				return true;
			}
			else //it is not your name? Make sure, asking your parent :)
				return shape::is_of(cls);
		}

		//Default implementation of the rendering: linear text...
		virtual handle render(request &req)
		{
			return forward_render((req_geometry &) req);
		}

	protected:
		// handlers of the requests - non-virtual, because virtual is 
		// the on_request() function, itself. 
		handle forward_render(req_geometry &req)
		{
			if (!display()) return skip(req);
			//preserve received (bounds) transformation and style
			transform	trans	= req.trans;	
			styleset	style	= req.style;
			//change the request, according styles and transformation of this group
			req.trans.premultiply(m_trans);		
			req.style.update(req,m_attribute_first, m_attribute_last);
			//forward request
			handle result = shape::forward(req);
			//restore previous state of the request - (bounds) transformation and style
			req.trans	= trans; 
			req.style	= style;

			//return the result, received from the descendants
			return result;
		}
}; //group

}; //namespace svg
#endif SVG_GROUP_INCLUDED
