//----------------------------------------------------------------------------
// SVG Viewer - Version 1.0
// A library of SVG renderer
// Copyright (C) 2008 Witold Jaworski
// Contact:	witold.jaworski@airplanes3d.net
//			http://airplanes3d.net
// 
// Contains elements from:
//			AGG - a graphic library (Copyright Maxim Shemanarev, mcseem@antigrain.com)
//          http://antigrain.com
// uses functions from:
//          Expat - a XML parser library (Copyright James Clark)
// 
// SVG Viewer is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// SVG Viewer is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with SVG Viewer; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
// MA 02110-1301, USA.
//----------------------------------------------------------------------------

#ifndef SVG_LINE_INCLUDED
#include "svg_path.h"
#define SVG_LINE_INCLUDED

namespace svg
{
//////////////////////////////////////////////////////////////////////
// line: draws a single line segment
//////////////////////////////////////////////////////////////////////
class line : public path
{

	public:
		//////////////////////////////////////////////////////////////////////
		// constructor and overriden virtual methods
		//////////////////////////////////////////////////////////////////////

		line(element *parent, unsigned level): path(parent, level), 
		m_x1(0), m_y1(0), m_x2(0), m_y2(0)
		{}
	
		virtual const char * tag() {return "line";}
		

		virtual void on_attribute(parser &prs, const char * name, const char *value)
		{
			if (strcmp(name, "x1") == 0) 
			{	
				m_x1 = prs.to_length(value); //throws an exception for percents
			}
			else
			if (strcmp(name, "y1") == 0) 
			{	
				m_y1 = prs.to_length(value); //throws an exception for percents
			}
			else
			if (strcmp(name, "x2") == 0) 
			{	
				m_x2 = prs.to_length(value); //throws an exception for percents
			}
			else
			if (strcmp(name, "y2") == 0) 
			{	
				m_y2 = prs.to_length(value); //throws an exception for percents
			}
			else
				path::on_attribute(prs,name,value);
		}//on_attribute
		
		//We are able to create the path for this line, when the end of the 
		//XML source element is reached:
		virtual void commit(parser &prs) 
		{
			m_path = prs.create_line(m_x1, m_y1, m_x2, m_y2);
			path::commit(prs);
		}

	protected:
		//////////////////////////////////////////////////////////////////////
		// Instance additional fields
		//////////////////////////////////////////////////////////////////////
		double m_x1, m_y1;	//position of the begining
		double m_x2, m_y2;	//position of the end
}; //line

}; //namespace svg
#endif SVG_LINE_INCLUDED
