//----------------------------------------------------------------------------
// SVG Viewer - Version 1.0
// A library of SVG renderer
// Copyright (C) 2008 Witold Jaworski
// Contact:	witold.jaworski@airplanes3d.net
//			http://airplanes3d.net
// 
// Contains elements from:
//			AGG - a graphic library (Copyright Maxim Shemanarev, mcseem@antigrain.com)
//          http://antigrain.com
// Uses functions from:
//          Expat - a XML parser library (Copyright James Clark)
// 
// SVG Viewer is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// SVG Viewer is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with SVG Viewer; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
// MA 02110-1301, USA.
//----------------------------------------------------------------------------

#ifndef SVG_LINEAR_GRADIENT_INCLUDED
#include "svg_gradient.h"
#define SVG_LINEAR_GRADIENT_INCLUDED

namespace svg
{

//----------------------------- Offset -------------------------------
//////////////////////////////////////////////////////////////////////
// linear_gradient: one of the two gradient types, used in SVG. 
//			        (the second one is the radial_gradient)
//////////////////////////////////////////////////////////////////////
class linear_gradient : public gradient 
{
	public:
		//////////////////////////////////////////////////////////////////////
		// constructor and overriden virtual methods
		//////////////////////////////////////////////////////////////////////

		linear_gradient(element *parent, unsigned level): gradient(parent, level),
		m_x1(0.0), m_y1(0.0), m_x2(0.0), m_y2(0.0)
		{}

		virtual const char * tag() {return "linear_gradient";};

		virtual void on_attribute(parser &prs, const char * name, const char *value)
		{
			if (strcmp(name, "x1") == 0) 
			{	
				m_x1 = parse_coordinate(prs,value);
			}
			else 
			if (strcmp(name, "y1") == 0) 
			{	
				m_y1 = parse_coordinate(prs,value);
			}
			else 
			if (strcmp(name, "x2") == 0) 
			{	
				m_x2 = parse_coordinate(prs,value);
			}
			else 
			if (strcmp(name, "y2") == 0) 
			{	
				m_y2 = parse_coordinate(prs,value);
			}
			else 
			{//if the style is not recognized:
				gradient::on_attribute(prs,name,value);
			}
		}
		
		virtual bool is_of(const char *cls) 
		{
			if(strcmp("linear_gradient",cls) == 0)
			{
				return true;
			}
			else //it is not your name? Make sure, asking your parent :)
				return gradient::is_of(cls);
		}

	protected:
		//Initializes the gradient, when the object receives a reqest::initialize
		//Arguments:
		//	req:	req::initialize - passed to give the access to the element and
		//			attribute storage.	
		//
		// BEWARE: in the child classes initialize the m_span, before calling this one! 
		//
		virtual void initialize(request &req)
		{
			m_span = segment(m_x1, m_y1, m_x2, m_y2); //since this early moment.
													  //do not in this class the
													  //m_x1, m_y1, m_x2, m_y2 -
													  //use m_span instead!
			gradient::initialize(req);

		}

		//Returns an offset generator object matching the given symbol
		//Arguments:
		//	symbol:	one of three values: "repeat", "reflect", "pad"
		//Each child class should return a correspodnig offset generator
		virtual svg::offset_generator *select_generator(const char *symbol)
		{
			if(strcmp(symbol, "reflect") == 0)
			{
				return &m_src_reflect;
			}
			else
			if(strcmp(symbol, "repeat") == 0)
			{
				return &m_src_repeat;
			}
			else //default: pad
			{
				return &m_src_pad;
			}
		}


	protected:
	protected:
		template<class GradientAdaptor> 
		class offset_source : public svg::offset_generator
		{
			public:
			offset_source() : m_adaptor(m_function) {}

			virtual int calculate(int x, int y, int d) const
			{
				return m_adaptor.calculate(x, y, d);
			}

			protected:
			agg::gradient_x m_function;
			GradientAdaptor m_adaptor;
		};//end offset_source

		typedef agg::gradient_reflect_adaptor<agg::gradient_x>	reflect_adaptor;
		typedef agg::gradient_repeat_adaptor<agg::gradient_x>	repeat_adaptor;
		typedef gradient_clamp_adaptor<agg::gradient_x>			pad_adaptor;

		//////////////////////////////////////////////////////////////////////
		// Instance additional fields
		//////////////////////////////////////////////////////////////////////
		//Gradient coordinates
		double m_x1;
		double m_y1;
		double m_x2;
		double m_y2;

		//Alternative offset sources:
		 offset_source<reflect_adaptor>	m_src_reflect;
		 offset_source<repeat_adaptor>	m_src_repeat;
		 offset_source<pad_adaptor>		m_src_pad;
}; //linear_gradient

}; //namespace svg
#endif SVG_LINEAR_GRADIENT_INCLUDED
