//----------------------------------------------------------------------------
// SVG Viewer - Version 1.0
// A library of SVG renderer
// Copyright (C) 2008 Witold Jaworski
// Contact:	witold.jaworski@airplanes3d.net
//			http://airplanes3d.net
// 
// Contains elements from:
//			AGG - a graphic library (Copyright Maxim Shemanarev, mcseem@antigrain.com)
//          http://antigrain.com
// Uses functions from:
//          Expat - a XML parser library (Copyright James Clark)
// 
// SVG Viewer is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// SVG Viewer is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with SVG Viewer; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
// MA 02110-1301, USA.
//----------------------------------------------------------------------------

#ifndef SVG_OPACITY_INCLUDED
#include "svg_filter.h"
#define SVG_OPACITY_INCLUDED

namespace svg
{
//////////////////////////////////////////////////////////////////////
// opacity: in fact, a specialized filter, which just copies the rendered
//			primitive to the destination with opacity specified.
//			(no fe_* primitives are needed here)
//////////////////////////////////////////////////////////////////////
class opacity : public filter 
{
	public:
		//////////////////////////////////////////////////////////////////////
		// constructor and overriden virtual methods
		//////////////////////////////////////////////////////////////////////
		//Arguments:
		//  parent:		the parent element in the SVG Graph
		//	level:		level of the hierarchy in the SVG Graph
		//	op:			value of the transparency (cover_full = opaque).
		opacity(element * parent, unsigned level, agg::int8u op): filter(parent, level)
		{
			m_opacity = op;
			m_relative_units = 1;
		}

		//Creates an unnamed opacity filter as the child of the <parent> element,
		//and returns its handle
		//Arguments:
		//	prs:		actual parser
		//  parent:		the parent element in the SVG Graph
		//	level:		level of the hierarchy in the SVG Graph
		//	op:			value of the opacity (1.0 = opaque).
		static handle create(parser &prs, element *parent, unsigned level, double op)
		{
			agg::int8u opc = agg::uround(op*agg::cover_full);
			if(opc < agg::cover_full) //made opacity only when it is reasonable!
			{
				element *elm = new opacity(parent, level, opc);
				if(elm)
				{
					prs.store(*elm, "", false, false);
					elm->commit(prs);
					return elm->id();
				}
			}//end if opc < cover_full
			//general "else:"
			return invalid_handle;
		}

		//no extent is needed:
		virtual double extent_ratio() {return 1.0;}

		virtual const char * tag() {return "opacity";}; 

		//Renders the given elmement onto the result buffer, pointed by req
		//Arguments:
		//	elm:	element, which picture will be filtered and applied to the 
		//			result;
		//	req:	actual request to render 
		void perform(shape &elm, req_render &req)
		{
			if (m_opacity > 0)
			{
				filter::perform(elm,req,m_opacity);
			}
			//else - nothing to do, object is completly transparent!
		}

		
	protected:
		agg::int8u m_opacity;

}; //opacity

}; //namespace svg
#endif SVG_OPACITY_INCLUDED
