//----------------------------------------------------------------------------
// SVG Viewer - Version 1.0
// A library of SVG renderer
// Copyright (C) 2008 Witold Jaworski
// Contact:	witold.jaworski@airplanes3d.net
//			http://airplanes3d.net
// 
// Contains elements from:
//			AGG - a graphic library (Copyright Maxim Shemanarev, mcseem@antigrain.com)
//          http://antigrain.com
// uses functions from:
//          Expat - a XML parser library (Copyright James Clark)
// 
// SVG Viewer is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// SVG Viewer is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with SVG Viewer; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
// MA 02110-1301, USA.
//----------------------------------------------------------------------------

#ifndef SVG_RECT_INCLUDED
#include "svg_path.h"
#define SVG_RECT_INCLUDED

namespace svg
{
//////////////////////////////////////////////////////////////////////
// rect: draws a rectangle (with optional rounded corners)
//////////////////////////////////////////////////////////////////////
class rect : public path
{

	public:
		//////////////////////////////////////////////////////////////////////
		// constructor and overriden virtual methods
		//////////////////////////////////////////////////////////////////////

		rect(element *parent, unsigned level): path(parent, level), 
		m_x(0), m_y(0), m_width(0), m_height(0), m_rx(0), m_ry(0)
		{}
	
		virtual const char * tag() 	{ return "ellipse";	}
		

		virtual void on_attribute(parser &prs, const char * name, const char *value)
		{
			if (strcmp(name, "x") == 0) 
			{	
				m_x = prs.to_length(value); //throws an exception for percents
			}
			else
			if (strcmp(name, "y") == 0) 
			{	
				m_y = prs.to_length(value); //throws an exception for percents
			}
			else
			if (strcmp(name, "rx") == 0) 
			{	
				m_rx = prs.to_length(value); //throws an exception for percents
			}
			else
			if (strcmp(name, "ry") == 0) 
			{	
				m_ry = prs.to_length(value); //throws an exception for percents
			}
			else
			if (strcmp(name, "width") == 0) 
			{	
				m_width = prs.to_length(value); //throws an exception for percents
			}
			else
			if (strcmp(name, "height") == 0) 
			{	
				m_height = prs.to_length(value); //throws an exception for percents
			}
			else
				path::on_attribute(prs,name,value);
		}//on_attribute
		
		//We are able to create the path for this shape, when the end of the 
		//XML source element is reached:
		virtual void commit(parser &prs) 
		{
			m_path = prs.create_rect(m_x, m_y, m_width, m_height, m_rx, m_ry);
			path::commit(prs);
		}

	protected:
		//////////////////////////////////////////////////////////////////////
		// Instance additional fields
		//////////////////////////////////////////////////////////////////////
		double m_x, m_y;	//upper left corner of the rectangle (without rounds)
		double m_width, m_height; //width and height of the rectangle
		double m_rx, m_ry;	//corner radii
}; //line

}; //namespace svg
#endif SVG_RECT_INCLUDED
