//----------------------------------------------------------------------------
// SVG Viewer - Version 1.0
// A library of SVG renderer
// Copyright (C) 2008 Witold Jaworski
// Contact:	witold.jaworski@airplanes3d.net
//			http://airplanes3d.net
// 
// Contains elements from:
//			AGG - a graphic library (Copyright Maxim Shemanarev, mcseem@antigrain.com)
//          http://antigrain.com
// Uses functions from:
//          Expat - a XML parser library (Copyright James Clark)
// 
// SVG Viewer is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// SVG Viewer is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with SVG Viewer; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
// MA 02110-1301, USA.
//----------------------------------------------------------------------------

#ifndef SVG_TEXT_PATH_INCLUDED
#include "svg_text.h"
#include "svg_path_text_position.h"
#define SVG_TEXT_PATH_INCLUDED

namespace svg
{
//////////////////////////////////////////////////////////////////////
// text_path: a text on a path (<textPath>) content in the SVG graph
//////////////////////////////////////////////////////////////////////
class text_path : public text
{
	public:
		//////////////////////////////////////////////////////////////////////
		// constructor and overriden virtual methods
		//////////////////////////////////////////////////////////////////////

		text_path(element *parent, unsigned level): text(parent, level) 
		{
			m_offset = 0.0;
			m_href = invalid_handle;
		}

		virtual const char * tag() {return "textPath";};

		virtual void on_attribute(parser &prs, const char * name, const char *value)
		{
			if (
					(strcmp(name, "startOffset") == 0) 
			   )
			{	
				bool percent = false;
				m_offset  = prs.to_length(value, &percent);
				if (percent) m_offset *= -1.0; //relative value is marked in this case
											   //as the NEGATIVE value
			}
			else 
			if (
					(strcmp(name, "xlink:href") == 0) 
			   )
			{	
				m_href  = prs.get_id(value); //id of the path
			}
			else 
			{//if the style is not recognized:
				text::on_attribute(prs,name,value);
			}
		}//on_attribute


		//Default implementation of the rendering: linear text...
		virtual handle render(request &rreq)
		{
			if (!display()) return skip(rreq);

			req_geometry &req = (req_geometry &) rreq;

			path *p = (path *) req.get_element(m_href,"path");

			if (p)
			{
				path_text_position tpos(p->vertices(req), p->path_id(), 
											m_offset, p->transformation());
				return text::render(req, tpos);
			}
			else return skip(req);
		}
		
	protected:
		//////////////////////////////////////////////////////////////////////
		// Instance additional fields
		//////////////////////////////////////////////////////////////////////
		double m_offset; //start offset
		handle m_href;
		

}; //text

}; //namespace svg
#endif SVG_TEXT_PATH_INCLUDED
