//----------------------------------------------------------------------------
// SVG Viewer - Version 1.0
// A library of SVG renderer
// Copyright (C) 2008 Witold Jaworski
// Contact:	witold.jaworski@airplanes3d.net
//			http://airplanes3d.net
// 
// Contains elements from:
//			AGG - a graphic library (Copyright Maxim Shemanarev, mcseem@antigrain.com)
//          http://antigrain.com
// Uses functions from:
//          Expat - a XML parser library (Copyright James Clark)
// 
// SVG Viewer is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// SVG Viewer is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with SVG Viewer; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
// MA 02110-1301, USA.
//----------------------------------------------------------------------------

#ifndef TEX_SRC_INCLUDED
#include "svg_group.h"
#define TEX_SRC_INCLUDED

namespace tex
{
//////////////////////////////////////////////////////////////////////
// tex::src: element NOT from the SVG spec!
// You can ignore it. It is used in an xml files that defines detail
// of the Blender svg textures. Used to point, which svg file to load
//	
//////////////////////////////////////////////////////////////////////
class src : public svg::group 
{
	public:
		//////////////////////////////////////////////////////////////////////
		// constructor and overriden virtual methods
		//////////////////////////////////////////////////////////////////////

		src(svg::element *parent, unsigned level): group(parent, level) 
		{memset(m_filename,0,(sizeof(m_filename)/sizeof(m_filename[0]))-1);}

		virtual const char * tag() {return "tex:src";};

		virtual void on_attribute(svg::parser &prs, const char * name, const char *value)
		{
			if (strcmp(name, "path") == 0) 
			{
				strncpy(m_filename, value, sizeof(m_filename)-1);
			}
			else
			{
				group::on_attribute(prs,name,value);
			}

		}//on_attribute

		virtual void on_content(svg::parser &prs, const char* s, int len) 
		{
			if (!commited()) commit(prs); //let it parse svg picture before reading 
										  //the content (the content of xml configu-
										  //ration file, in fact - subelements of this
										  //tex:src element, like tex:override)	
			group::on_content(prs, s, len);
		}
		//on commit: load the referenced file 
		//(at this final moment, because *this element should be already stored)
		virtual void commit(svg::parser &prs) 
		{
			if (!commited()) prs.parse(m_filename); //we can parse svg picture only once!
			group::commit(prs);
		}

		//////////////////////////////////////////////////////////////////////
		// Other methods and fields
		//////////////////////////////////////////////////////////////////////
	protected:
		char m_filename[SVG_MAX_ATTRIB_VALUE*4]; //*4, because it will be safer
												 // for a very long file names!
}; //src

}; //namespace tex
#endif TEX_SRC_INCLUDED
